@@ -22,12 +22,14 @@
 #define UART_U2X		(UART_BAUD_U2X(4) <= 4095)
 #define UART_BAUD_SCALE		(UART_U2X ? UART_BAUD_U2X(4) : UART_BAUD_U2X(8))
 
-#define RINGSIZE	16
+#ifndef UART_RINGSIZE
+#define UART_RINGSIZE	16
+#endif
 
 typedef volatile struct uart_ring {
 	uint8_t	read;
 	uint8_t	count;
-	uint8_t	buf[RINGSIZE];
+	uint8_t	buf[UART_RINGSIZE];
 } uart_ring_t;
 
 static uart_ring_t	rx_ring, tx_ring;
@@ -40,9 +42,33 @@ static volatile uint8_t	tx_flow;
 #define FLOW_FULL	2
 #define FLOW_STOPPED	3
 
+#if defined(__AVR_ATmega640__) || defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__) || defined(__AVR_ATmega1284__)
+#define UDRE_vect USART0_UDRE_vect
+#define RX_vect USART0_RX_vect
+#elif defined(__AVR_ATmega4809__)
+# define UDRE_vect 	USART3_DRE_vect
+# define RX_vect 	USART3_RXC_vect
+# define DISABLE_DRE()	(USART3.CTRLA &= ~USART_DREIE_bm)
+# define ENABLE_DRE()	(USART3.CTRLA |= USART_DREIE_bm)
+# define TX_EMPTY()	(USART3.STATUS & USART_DREIF_bm)
+# define RX_DATA	USART3.RXDATAL
+# define TX_DATA	USART3.TXDATAL
+#else
+#define UDRE_vect USART_UDRE_vect
+#define RX_vect USART_RX_vect
+#endif
+
+#ifndef DISABLE_DRE
+#define RX_DATA	UDR0
+#define TX_DATA UDR0
+#define DISABLE_DRE()	(UCSR0B &= ~(1 << UDRIE0))
+#define ENABLE_DRE()	(UCSR0B |= (1 << UDRIE0))
+#define TX_EMPTY()	(UCSR0A & (1 << UDRE0))
+#endif
+
 static bool ring_full(uart_ring_t *ring)
 {
-	return ring->count == RINGSIZE;
+	return ring->count == UART_RINGSIZE;
 }
 
 static bool ring_empty(uart_ring_t *ring)
@@ -52,7 +78,7 @@ static bool ring_empty(uart_ring_t *ring)
 
 static bool ring_mostly_full(uart_ring_t *ring)
 {
-	return ring->count >= (RINGSIZE / 2);
+	return ring->count >= (UART_RINGSIZE / 2);
 }
 
 static int
@@ -61,7 +87,7 @@ ring_get(uart_ring_t *ring)
 	if (ring_empty(ring))
 		return -1;
 	uint8_t c = ring->buf[ring->read];
-	ring->read = (ring->read + 1) & (RINGSIZE - 1);
+	ring->read = (ring->read + 1) & (UART_RINGSIZE - 1);
 	ring->count--;
 	return c;
 }
@@ -71,7 +97,7 @@ ring_put(uart_ring_t *ring, uint8_t c)
 {
 	if (ring_full(ring))
 		return false;
-	uint8_t write = (ring->read + ring->count) & (RINGSIZE - 1);
+	uint8_t write = (ring->read + ring->count) & (UART_RINGSIZE - 1);
 	ring->buf[write] = c;
 	ring->count++;
 	return true;
@@ -86,7 +112,7 @@ next_flow(void)
 static void
 _snek_uart_tx_start(void)
 {
-	if ((UCSR0A & (1 << UDRE0)) && !tx_flow) {
+	if (TX_EMPTY() && !tx_flow) {
 		uint8_t c;
 
 		if ((rx_flow & 1) == 0) {
@@ -102,8 +128,8 @@ _snek_uart_tx_start(void)
 				return;
 			c = ic;
 		}
-		UCSR0B |= (1 << UDRIE0);
-		UDR0 = c;
+		ENABLE_DRE();
+		TX_DATA = c;
 	}
 }
 
@@ -128,24 +154,16 @@ _snek_uart_xoff(void)
 		_snek_uart_flow_do();
 }
 
-#if defined(__AVR_ATmega640__) || defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)
-#define UDRE_vect USART0_UDRE_vect
-#define RX_vect USART0_RX_vect
-#else
-#define UDRE_vect USART_UDRE_vect
-#define RX_vect USART_RX_vect
-#endif
-
 ISR(UDRE_vect)
 {
-	UCSR0B &= ~(1 << UDRIE0);
+	DISABLE_DRE();
 	_snek_uart_xon();
 	_snek_uart_tx_start();
 }
 
 ISR(RX_vect)
 {
-	uint8_t	c = UDR0;
+	uint8_t	c = RX_DATA;
 
 	switch (c) {
 	case 'c' & 0x1f:
@@ -185,9 +203,6 @@ void
 snek_uart_putch(char c)
 {
 	switch (c) {
-	case 'q' & 0x1f:
-	case 's' & 0x1f:
-		return;
 	case '\n':
 		snek_uart_putch('\r');
 		break;
@@ -222,6 +237,22 @@ _snek_uart_puts(const char *PROGMEM string)
 void
 snek_uart_init(void)
 {
+#if defined(__AVR_ATmega4809__)
+	PORTMUX.USARTROUTEA |= PORTMUX_USART30_bm;
+
+	int32_t baud_setting;
+	baud_setting = (((8 * F_CPU) / UART_BAUD) + 1) / 2;
+	int8_t sigrow_val = SIGROW.OSC16ERR5V;
+	baud_setting += (baud_setting * sigrow_val) / 1024;
+
+	USART3.BAUD = baud_setting;
+	USART3.CTRLC = (USART_CHSIZE0_bm | USART_CHSIZE1_bm);
+	VPORTB_DIR |= (1 << 4);
+	VPORTB_OUT |= (1 << 4);
+	USART3.CTRLB = (USART_RXEN_bm |
+			USART_TXEN_bm);
+	USART3.CTRLA = USART_RXCIE_bm;
+#else
 	UBRR0H = (uint8_t) (UART_BAUD_SCALE >> 8);
 	UBRR0L = (uint8_t) (UART_BAUD_SCALE);
 	UCSR0A = ((1 << TXC0) |
@@ -242,5 +273,6 @@ snek_uart_init(void)
 		  (1 << TXEN0) |
 		  (0 << UCSZ02) |
 		  (0 << TXB80));
+#endif
 	snek_uart_puts("Welcome to Snek " SNEK_VERSION "\n");
 }
