@@ -18,6 +18,10 @@ import curses
 import serial
 import serial.tools.list_ports
 import sys
+
+termios_error = serial.SerialException
+if hasattr(serial, "termios"):
+    termios_error = termios.error
 import threading
 import time
 
@@ -27,7 +31,6 @@ from pathlib import Path, PurePath
 stdscr = 0
 
 snek_lock = threading.Lock()
-snek_lock.acquire()
 
 snek_current_window = 0
 snek_edit_win = 0
@@ -44,11 +47,18 @@ snek_dialog_waiting = False
 
 snek_cur_path = None
 
-# snek_debug_file = open('log', 'w')
+snek_debug_file = False
 
-# def snek_debug(message):
-#    snek_debug_file.write(message + '\n')
-#    snek_debug_file.flush()
+
+def snek_debug(message):
+    global snek_debug_file
+    if not snek_debug_file:
+        snek_debug_file = open("log", "w")
+    snek_debug_file.write(message + "\n")
+    snek_debug_file.flush()
+
+
+# snek_debug("Starting snekde")
 
 #
 # Read a character from the keyboard, releasing the
@@ -60,11 +70,13 @@ def my_getch(edit_win):
     global snek_lock, snek_current_window, snek_dialog_waiting
     while True:
         edit_win.set_cursor()
-        snek_lock.release()
-        c = edit_win.window.getch()
+        try:
+            snek_lock.release()
+            c = edit_win.window.getch()
+        finally:
+            snek_lock.acquire()
         if c == ord("\r"):
             c = ord("\n")
-        snek_lock.acquire()
         if not snek_dialog_waiting:
             break
         if c == ord("\n"):
@@ -95,7 +107,8 @@ class SnekPort:
                     self.name = self.name[self.name.rfind("/") + 1 :]
             else:
                 self.name = port[1]
-            self.description = port[2]
+            self.description = port[1]
+            self.hwid = port[2]
         else:
             self.device = port.device
             if port.name:
@@ -103,6 +116,8 @@ class SnekPort:
             else:
                 self.name = self.device
             self.description = port.description
+            self.product = port.product
+            self.hwid = port.hwid
         if self.name and "/" in self.name:
             self.name = self.name[self.name.rfind("/") + 1 :]
 
@@ -147,6 +162,9 @@ def snek_list_ports():
     return ports
 
 
+port_mods = {"239A:": "async"}
+
+
 class SnekDevice:
     """Link to snek device"""
 
@@ -159,6 +177,7 @@ class SnekDevice:
     write_queue = False
     device = ""
     interrupt_pending = False
+    syncronous_put = False
 
     #
     # The interface needs to have a condition variable (cv) that is
@@ -169,16 +188,25 @@ class SnekDevice:
     def __init__(self, port, interface):
         self.interface = interface
         self.device = port.device
+
+        # For devices without flow control, use snek's ENQ/ACK
+        # mechanism to avoid overrunning the device input buffer
+
+        self.synchronous_put = True
+        self.synchronous_limit = 16
+
         rate = 115200
-        if "Arduino Mega" in port.description:
-            rate = 38400
+        for port_mod in port_mods:
+            if port_mod in port.description or port_mod in port.hwid:
+                if "async" in port_mods[port_mod]:
+                    self.synchronous_put = False
         self.serial = serial.Serial(
             port=self.device,
             baudrate=rate,
             bytesize=serial.EIGHTBITS,
             parity=serial.PARITY_NONE,
             stopbits=serial.STOPBITS_ONE,
-            xonxoff=True,
+            xonxoff=False,
             rtscts=False,
             dsrdtr=False,
         )
@@ -225,7 +253,6 @@ class SnekDevice:
             self.serial.write_timeout = 1
             self.serial.write(b"\x0f")
             self.serial.reset_output_buffer()
-            self.serial.xonxoff = False
             self.serial.close()
         except serial.SerialException:
             pass
@@ -237,54 +264,84 @@ class SnekDevice:
                 # read all that is there or wait for one byte
                 data = self.serial.read(1)
                 if data:
-                    self.interface.receive(str(data, encoding="utf-8", errors="ignore"))
-        except serial.SerialException as e:
+                    self.interface.receive(data)
+        except (serial.SerialException, termios_error):
+            # snek_debug('serial interface failed in reader')
             self.interface.failed(self.device)
         finally:
             self.receiver_thread = False
 
     def writer(self):
+        global snek_lock
+
         """Copy queued data to the serial port."""
         try:
-            while self.alive:
-                send_data = ""
+            sent = 0
+            while True:
+                send_data = b""
                 interrupt = False
-                with self.interface.cv:
+                # snek_debug('writer locking')
+                with snek_lock:
+                    # snek_debug('writer waiting')
+
+                    # Wait until there's something to do
+
                     while (
                         not self.write_queue
                         and not self.interrupt_pending
                         and self.alive
                     ):
                         self.interface.cv.wait()
-                    if not self.alive:
-                        return
+                    # snek_debug('writer awake')
                     send_data = self.write_queue
                     interrupt = self.interrupt_pending
                     self.write_queue = False
                     self.interrupt_pending = False
+                if not self.alive:
+                    break
                 if interrupt:
+                    # snek_debug('interrupt')
                     self.serial.reset_output_buffer()
-                    self.serial.xonxoff = False
                     self.serial.write(b"\x0f\x03\x0e")
-                    self.serial.xonxoff = True
-                if send_data:
-                    self.serial.write(send_data.encode("utf-8"))
-        except (serial.SerialException, termios.error):
+                if self.synchronous_put:
+                    while send_data:
+                        amt = self.synchronous_limit - sent
+                        if amt > len(send_data):
+                            amt = len(send_data)
+                        this_time = send_data[:amt]
+                        send_data = send_data[amt:]
+                        self.serial.write(this_time)
+                        sent += amt
+                        if sent >= self.synchronous_limit:
+                            # snek_debug('sync %d limit %d' % (sent, self.synchronous_limit))
+                            snek_monitor.sync()
+                            sent = 0
+                else:
+                    if send_data:
+                        self.serial.write(send_data)
+        except (serial.SerialException, termios_error):
+            # snek_debug('serial interface failed in writer')
             self.interface.failed(self.device)
         finally:
             self.transmitter_thread = False
 
     def interrupt(self):
+        # snek_debug('pend interrupt waited')
         self.interrupt_pending = True
         self.interface.cv.notify()
+        # snek_debug('pended')
 
-    def write(self, data):
+    def writeb(self, data):
+        # snek_debug('writeb waited')
         if self.write_queue:
             self.write_queue += data
         else:
             self.write_queue = data
         self.interface.cv.notify()
 
+    def write(self, data):
+        self.writeb(data.encode("utf-8"))
+
     def command(self, data, intr="\x03"):
         self.write("\x0e" + intr + data)
 
@@ -759,6 +816,7 @@ class ErrorWin:
     def __init__(self, label, inputthread=True):
         self.label = label
         self.inputthread = inputthread
+        self.nlines = 5
         self.ncols = min(snek_cols, max(40, len(label) + 2))
         self.x = (snek_cols - self.ncols) // 2
         self.y = (snek_lines - self.nlines) // 2
@@ -1086,8 +1144,7 @@ def screen_paint():
 
 
 def screen_repaint():
-    global snek_edit_win, snek_repl_win
-    stdscr.clear()
+    global snek_edit_win, snek_repl_win, snek_current_window
     snek_edit_win.repaint()
     snek_repl_win.repaint()
     screen_paint()
@@ -1161,12 +1218,15 @@ def snekde_get_text():
 
 
 def snekde_put_text():
-    global snek_edit_win, snek_device
+    global snek_edit_win, snek_device, snek_monitor
     if len(snek_edit_win.text.strip()) == 0:
         ErrorWin("No program to put")
         return
     snek_device.command("eeprom.write()\n")
-    snek_device.write(snek_edit_win.text + "\x04")
+    text = snek_edit_win.text.encode("utf-8")
+    snek_device.writeb(text)
+    snek_device.writeb(b"\004")
+
     snek_device.command("reset()\n", intr="")
     snek_edit_win.changed = False
 
@@ -1179,6 +1239,7 @@ def load_file(path):
     with open(path, "r") as myfile:
         data = myfile.read()
         snek_edit_win.set_text(data)
+        snek_edit_win.repaint()
         snek_cur_path = path
         snek_edit_win.changed = False
 
@@ -1229,9 +1290,11 @@ def snekde_save_file():
 
 
 def run():
-    global snek_current_window, snek_edit_win, snek_repl_win, snek_device
+    global snek_current_window, snek_edit_win, snek_repl_win, snek_device, snek_lock
     snek_current_window = snek_edit_win
     prev_exit = False
+    prev_get = False
+    snek_lock.acquire()
     while True:
         ch = snek_current_window.getch()
         if ch == curses.ascii.ESC:
@@ -1243,6 +1306,10 @@ def run():
             snekde_open_device()
         elif ch == curses.KEY_F2 or ch == ord("2") | 0x80:
             if snek_device:
+                if snek_edit_win.changed and not prev_get:
+                    ErrorWin("Unsaved changes, get again to abandon them")
+                    prev_get = True
+                    continue
                 snekde_get_text()
             else:
                 ErrorWin("No device")
@@ -1296,6 +1363,7 @@ def run():
         else:
             snek_current_window.dispatch(ch)
         prev_exit = False
+        prev_get = False
 
 
 # Class to monitor the serial device for data and
@@ -1309,11 +1377,23 @@ class SnekMonitor:
     def __init__(self):
         global snek_lock
         self.cv = threading.Condition(snek_lock)
+        self.ack = threading.Condition(snek_lock)
+        self.getting_text = False
+
+    # Synchronize with the remote device by sending ENQ and
+    # waiting until an ACK is received
+
+    def sync(self):
+        global snek_lock
+        # snek_debug('sync locking')
+        with snek_lock:
+            # snek_debug('sync locked')
+            snek_device.serial.write(b"\x05")
+            self.ack.wait(1)
+            # snek_debug('sync done')
 
     # Reading text to snek_edit_win instead of snek_repl_win
 
-    getting_text = False
-
     def add_to(self, window, data):
         global snek_current_window, snek_repl_win
         follow = window == snek_repl_win and window.point == len(window.text)
@@ -1324,37 +1404,66 @@ class SnekMonitor:
         if snek_current_window:
             snek_current_window.set_cursor()
 
+    # Receive serial data
+
     def receive(self, data):
         global snek_edit_win, snek_repl_win, snek_lock
-        data_edit = ""
-        data_repl = ""
+        data_edit = b""
+        data_repl = b""
         for c in data:
-            if c == "\x02":
+            b = bytes((c,))
+
+            # STX - start receiving eeprom contents
+            if b == b"\x02":
                 self.getting_text = True
-            elif c == "\x03":
+
+            # ETX - stop receiving eeprom contents
+            elif b == b"\x03":
                 self.getting_text = False
-            elif c == "\x00":
-                continue
-            elif c == "\r":
-                continue
+
+            # ACK - device has seen ENQ
+            elif b == b"\x06":
+                # snek_debug('receive ACK')
+                with snek_lock:
+                    # snek_debug('notify ack')
+                    self.ack.notify_all()
+
+            # Ignore all control chars other than newline
+            elif b < b"\x20" and b != b"\n":
+                pass
+
+            # Otherwise, handle text
             else:
                 if self.getting_text:
-                    data_edit += c
+                    data_edit += b
                 else:
-                    data_repl += c
+                    data_repl += b
+
+        # Append data to appropriate buffer
+
         with snek_lock:
             if data_edit:
-                self.add_to(snek_edit_win, data_edit)
+                self.add_to(
+                    snek_edit_win, str(data_edit, encoding="utf-8", errors="ignore")
+                )
             if data_repl:
-                self.add_to(snek_repl_win, data_repl)
+                self.add_to(
+                    snek_repl_win, str(data_repl, encoding="utf-8", errors="ignore")
+                )
+
+    # Device has failed in reader or writer thread
 
     def failed(self, device):
         global snek_device, snek_lock
+
+        # Need to acquire lock for other threads
+
         with snek_lock:
             if snek_device:
                 snek_device.close()
                 del snek_device
                 snek_device = False
+            # snek_debug('show error dialog')
             ErrorWin("Device %s failed" % device, inputthread=False)
 
 
@@ -1373,7 +1482,7 @@ def main():
     snek_device = False
     if args.list:
         for port in snek_list_ports():
-            print("%-30.30s %s" % (port.name, port.description))
+            print("%-30.30s %s %s" % (port.name, port.description, port.hwid))
         sys.exit(0)
     if args.port:
         try:
